/*  GtkScintilla: Wrapper widget for the Scintilla editing component.
 *
 * Copyright (c) 2001  Michele Campeotto <micampe@f2s.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __GTK_SCINTILLA_H__
#define __GTK_SCINTILLA_H__

#include <gtk/gtk.h>

#ifdef __cplusplus
extern "C" {
#endif				/* __cplusplus */
/* *INDENT-OFF* */
#define GTK_TYPE_SCINTILLA             (gtk_scintilla_get_type ())
#define GTK_SCINTILLA(obj)             (GTK_CHECK_CAST ((obj), GTK_TYPE_SCINTILLA, GtkScintilla))
#define GTK_SCINTILLA_CLASS(klass)     (GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_SCINTILLA, GtkScintillaClass))
#define GTK_IS_SCINTILLA(obj)          (GTK_CHECK_TYPE ((obj), GTK_TYPE_SCINTILLA))
#define GTK_IS_SCINTILLA_CLASS(klass)  (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_SCINTILLA))

typedef struct _GtkScintilla        GtkScintilla;
typedef struct _GtkScintillaClass   GtkScintillaClass;

typedef enum {
    GTK_SCINTILLA_STYLE_DEFAULT = 32,
    GTK_SCINTILLA_STYLE_LINE_NUMBER,
    GTK_SCINTILLA_STYLE_BRACE_LIGHT,
    GTK_SCINTILLA_STYLE_BRACE_BAD,
    GTK_SCINTILLA_STYLE_CONTROL_CHAR,
    GTK_SCINTILLA_STYLE_INDENT_GUIDE,
    GTK_SCINTILLA_STYLE_MAX = 127
} GtkScintillaStyle;

typedef enum {
    GTK_SCINTILLA_LEXER_CONTAINER,
    GTK_SCINTILLA_LEXER_NULL,
    GTK_SCINTILLA_LEXER_PYTHON,
    GTK_SCINTILLA_LEXER_CPP,
    GTK_SCINTILLA_LEXER_HTML,
    GTK_SCINTILLA_LEXER_XML,
    GTK_SCINTILLA_LEXER_PERL,
    GTK_SCINTILLA_LEXER_SQL,
    GTK_SCINTILLA_LEXER_VB,
    GTK_SCINTILLA_LEXER_PROPERTIES,
    GTK_SCINTILLA_LEXER_ERRORLIST,
    GTK_SCINTILLA_LEXER_MAKEFILE,
    GTK_SCINTILLA_LEXER_BATCH,
    GTK_SCINTILLA_LEXER_XCODE,
    GTK_SCINTILLA_LEXER_LATEX,
    GTK_SCINTILLA_LEXER_LUA
} GtkScintillaLexer;

typedef enum {
    GTK_SCINTILLA_EOL_CRLF,
    GTK_SCINTILLA_EOL_CR,
    GTK_SCINTILLA_EOL_LF
} GtkScintillaEolMode;

typedef enum {
    GTK_SCINTILLA_MARGIN_SYMBOL,
    GTK_SCINTILLA_MARGIN_NUMBER
} GtkScintillaMarginType;

typedef enum {
    GTK_SCINTILLA_EDGE_NONE,
    GTK_SCINTILLA_EDGE_LINE,
    GTK_SCINTILLA_EDGE_BACKGROUND
} GtkScintillaEdgeMode;

typedef enum {
    GTK_SCINTILLA_MARK_CIRCLE,
    GTK_SCINTILLA_MARK_ROUNDRECT,
    GTK_SCINTILLA_MARK_ARROW,
    GTK_SCINTILLA_MARK_SMALLRECT,
    GTK_SCINTILLA_MARK_SHORTARROW,
    GTK_SCINTILLA_MARK_EMPTY,
    GTK_SCINTILLA_MARK_ARROWDOWN,
    GTK_SCINTILLA_MARK_MINUS,
    GTK_SCINTILLA_MARK_PLUS
} GtkScintillaMarkerSymbols;

struct _GtkScintilla {
    GtkFrame parent;
    
    GtkWidget *scintilla;
    
    GtkAccelGroup *accel_group;
};

struct _GtkScintillaClass {
    GtkFrameClass parent_class;

    void (* style_needed)       (GtkWidget *w, gint position);
    void (* update_ui)          (GtkWidget *w);
    void (* char_added)         (GtkWidget *w, gint ch);
    void (* save_point_reached) (GtkWidget *w);
    void (* save_point_left)    (GtkWidget *w);
    void (* modify_attempt_ro)  (GtkWidget *w);
    void (* double_click)       (GtkWidget *w);
    void (* key)                (GtkWidget *w, gint ch, gint modifiers);
    void (* modified)           (GtkWidget *w);
    void (* macro_record)       (GtkWidget *w);
    void (* margin_click)       (GtkWidget *w, gint margin);
    void (* need_shown)         (GtkWidget *w);
};

guint       gtk_scintilla_get_type          (void);
GtkWidget*  gtk_scintilla_new               (void);

void        gtk_scintilla_set_accel_group   (GtkScintilla *sci,
                                             GtkAccelGroup *accel_group);

/* Text retrieval and modification */
void        gtk_scintilla_get_text          (GtkScintilla *sci,
                                             guint length, gchar *text);
void        gtk_scintilla_set_text          (GtkScintilla *sci, gchar *text);
void        gtk_scintilla_get_line          (GtkScintilla *sci,
                                             guint line, gchar *text);
void        gtk_scintilla_seplace_selection (GtkScintilla *sci, gchar *text);
void        gtk_scintilla_set_read_only     (GtkScintilla *sci);
void        gtk_scintilla_insert_text       (GtkScintilla *sci,
                                             guint position, gchar *text);
void        gtk_scintilla_clear_document_style(GtkScintilla *sci);
gchar       gtk_scintilla_get_char_at       (GtkScintilla *sci, guint position);

/* Standard commands */
void        gtk_scintilla_cut               (GtkScintilla *sci);
void        gtk_scintilla_copy              (GtkScintilla *sci);
void        gtk_scintilla_paste             (GtkScintilla *sci);
gboolean    gtk_scintilla_can_paste         (GtkScintilla *sci);

/* Undo and Redo */
void        gtk_scintilla_undo              (GtkScintilla *sci);
gboolean    gtk_scintilla_can_undo          (GtkScintilla *sci);
void        gtk_scintilla_empty_undo_buffer (GtkScintilla *sci);
void        gtk_scintilla_redo              (GtkScintilla *sci);
gboolean    gtk_scintilla_can_redo          (GtkScintilla *sci);
void        gtk_scintilla_begin_undo_action (GtkScintilla *sci);
void        gtk_scintilla_end_undo_action   (GtkScintilla *sci);

/* Selection and information */
gint        gtk_scintilla_get_text_length   (GtkScintilla *sci);
gint        gtk_scintilla_get_line_count    (GtkScintilla *sci);
gboolean    gtk_scintilla_get_modify        (GtkScintilla *sci);
void        gtk_scintilla_set_selection     (GtkScintilla* sci,
                                             guint start, guint end);
void        gtk_scintilla_get_selected_text (GtkScintilla *sci, gchar *text);
gint        gtk_scintilla_line_from_position(GtkScintilla *sci, guint position);
gint        gtk_scintilla_position_from_line(GtkScintilla *sci, guint line);
gint        gtk_scintilla_line_length       (GtkScintilla *sci, guint line);
gint        gtk_scintilla_get_length        (GtkScintilla *sci);
void        gtk_scintilla_set_current_pos   (GtkScintilla *sci, guint position);
gint        gtk_scintilla_get_current_pos   (GtkScintilla *sci);
void        gtk_scintilla_set_selection_start(GtkScintilla *sci,
                                              guint position);
gint        gtk_scintilla_get_selection_start(GtkScintilla *sci);
void        gtk_scintilla_set_selection_end (GtkScintilla *sci, guint position);
gint        gtk_scintilla_get_selection_end (GtkScintilla *sci);
void        gtk_scintilla_select_all        (GtkScintilla *sci);
void        gtk_scintilla_go_to_line        (GtkScintilla *sci, guint line);
void        gtk_scintilla_go_to_pos         (GtkScintilla *sci, guint position);
void        gtk_scintilla_set_h_scrollbar   (GtkScintilla *sci,
                                             gboolean visible);

/* Searching */
void        gtk_scintilla_search_anchor     (GtkScintilla *sci);
void        gtk_scintilla_search_next       (GtkScintilla *sci,
                                             guint flags, gchar *text);
void        gtk_scintilla_search_prev       (GtkScintilla *sci,
                                             guint flags, gchar *text);

/* Visible white space */
void        gtk_scintilla_set_space_visible (GtkScintilla *sci,
                                             gboolean visible);

/* Line endings */
void        gtk_scintilla_set_eol_mode      (GtkScintilla *sci,
                                             GtkScintillaEolMode mode);
void        gtk_scintilla_set_eol_visible   (GtkScintilla *sci,
                                             gboolean visible);

/* Styling */

/* Style definition */
void        gtk_scintilla_style_clear_all       (GtkScintilla *sci);
void        gtk_scintilla_style_reset_default   (GtkScintilla *sci);
void        gtk_scintilla_style_set_fore        (GtkScintilla *sci,
                                                 GtkScintillaStyle style,
                                                 gint color);
void        gtk_scintilla_style_set_back        (GtkScintilla *sci,
                                                 GtkScintillaStyle style,
                                                 gint color);
void        gtk_scintilla_style_set_bold        (GtkScintilla *sci,
                                                 GtkScintillaStyle style,
                                                 gboolean bold);
void        gtk_scintilla_style_set_italic      (GtkScintilla *sci,
                                                 GtkScintillaStyle style,
                                                 gboolean italic);
void        gtk_scintilla_style_set_size        (GtkScintilla *sci,
                                                 GtkScintillaStyle style,
                                                 gint size);
void        gtk_scintilla_style_set_font        (GtkScintilla *sci,
                                                 GtkScintillaStyle style,
                                                 gchar *fontname);

void        gtk_scintilla_set_sel_fore          (GtkScintilla *sci,
                                                 gboolean use_color,
                                                 gint color);
void        gtk_scintilla_set_sel_back          (GtkScintilla *sci,
                                                 gboolean use_color,
                                                 gint color);
void        gtk_scintilla_set_caret_fore        (GtkScintilla *sci, gint color);
void        gtk_scintilla_set_caret_period      (GtkScintilla *sci,
                                                 guint period);

/* Margins */
void        gtk_scintilla_set_margin_left       (GtkScintilla *sci,
                                                 guint width);
void        gtk_scintilla_set_margin_right      (GtkScintilla *sci,
                                                 guint width);

void        gtk_scintilla_set_margin_type       (GtkScintilla *sci,
                                                 guint margin,
                                                 GtkScintillaMarginType type);
void        gtk_scintilla_set_margin_width      (GtkScintilla *sci,
                                                 guint margin, guint width);
void        gtk_scintilla_set_margin_sensitive  (GtkScintilla *sci,
                                                 guint margin,
                                                 gboolean sensitive);

/* Other settings */
void        gtk_scintilla_set_buffered_draw     (GtkScintilla *sci,
                                                 gboolean buffered);
void        gtk_scintilla_set_tab_width         (GtkScintilla *sci,
                                                 guint width);
void        gtk_scintilla_set_indent_width      (GtkScintilla *sci,
                                                 guint width);
void        gtk_scintilla_set_use_tabs          (GtkScintilla *sci,
                                                 gboolean use_tabs);
void        gtk_scintilla_set_line_indentation  (GtkScintilla *sci,
                                                 guint line, guint indentation);
gint        gtk_scintilla_get_line_indentation  (GtkScintilla *sci,
                                                 guint line);
gint        gtk_scintilla_get_line_indent_position(GtkScintilla *sci,
                                                   guint line);
gint        gtk_scintilla_get_column            (GtkScintilla *sci,
                                                 guint position);
void        gtk_scintilla_grab_focus            (GtkScintilla *sci);

/* Brace highlighting */

/* Indentation guides */
void        gtk_scintilla_set_indent_visible    (GtkScintilla *sci,
                                                 gboolean visible);
void        gtk_scintilla_highlight_guide       (GtkScintilla *sci,
                                                 guint column);

/* Markers */
void        gtk_scintilla_marker_define         (GtkScintilla *sci,
                                                 gint markernumber,
                                                 GtkScintillaMarkerSymbols markersymbols);
void        gtk_scintilla_marker_set_fore       (GtkScintilla *sci,
                                                 gint markernumber,
                                                 gint colour);
void        gtk_scintilla_marker_set_back       (GtkScintilla *sci,
                                                 gint markernumber,
                                                 gint colour);
void        gtk_scintilla_marker_add            (GtkScintilla *sci,
                                                 guint line,
                                                 gint markernumber);
void        gtk_scintilla_marker_delete         (GtkScintilla *sci,
                                                 guint line,
                                                 gint markernumber);
void        gtk_scintilla_marker_delete_all     (GtkScintilla *sci,
                                                 gint markernumber);
gint        gtk_scintilla_marker_get            (GtkScintilla *sci, gint line);
gint        gtk_scintilla_marker_next           (GtkScintilla *sci,
                                                 gint line, gint mask);
gint        gtk_scintilla_marker_previous       (GtkScintilla *sci,
                                                 gint line, gint mask);

/* Indicators */

/* Autocompletion */

/* Callstips */
void        gtk_scintilla_calltip_show      (GtkScintilla *sci,
                                             guint pos,
                                             gchar *definition);
void        gtk_scintilla_calltip_cancel    (GtkScintilla *sci);
gboolean    gtk_scintilla_calltip_active    (GtkScintilla *sci);
gint        gtk_scintilla_calltip_pos_start (GtkScintilla *sci);
/*void        gtk_scintilla_calltip_set_hlt   (GtkScintilla *sci);*/
void        gtk_scintilla_calltip_set_back  (GtkScintilla *sci, gint color);

/* Keyboard commands */

/* Key bindings */
void        gtk_scintilla_clear_all_cmd_keys (GtkScintilla *sci);

/* Popup edit menu */

/* Macro recording */

/* Multiple views */
long int    gtk_scintilla_get_doc_pointer   (GtkScintilla *sci);
void        gtk_scintilla_set_doc_pointer   (GtkScintilla *sci, long int doc);
long int    gtk_scintilla_create_document   (GtkScintilla *sci);
void        gtk_scintilla_add_ref_document  (GtkScintilla *sci, long int doc);
void        gtk_scintilla_release_document  (GtkScintilla *sci, long int doc);

/* Folding */

/* Zooming */

/* Long lines */
void        gtk_scintilla_set_edge_column   (GtkScintilla *sci, guint column);
void        gtk_scintilla_set_edge_mode     (GtkScintilla *sci,
                                             GtkScintillaEdgeMode mode);
void        gtk_scintilla_set_edge_color    (GtkScintilla *sci,
                                             gint color);

/* Lexer */
void        gtk_scintilla_set_lexer         (GtkScintilla *sci,
                                             GtkScintillaLexer lexer);
void        gtk_scintilla_colourise         (GtkScintilla *sci,
                                             guint start, gint end);
void        gtk_scintilla_set_property      (GtkScintilla *sci,
                                             gchar *key, gchar *value);
void        gtk_scintilla_set_keywords      (GtkScintilla *sci,
                                             gint keywordset, gchar *keywords);

/* Notifications */
void        gtk_scintilla_set_save_point    (GtkScintilla *sci);

/* *INDENT-ON* */
#
#ifdef __cplusplus
}
#endif				/* __cplusplus */
#endif				/* __GTK_SCINTILLA_H__ */
